# -*- coding: utf-8 -*-
import inspect
import re
from DjangoCaptcha import Captcha
from django import forms
from django.contrib.auth.models import User
from bbs.models import TopicType, Forum, Topic


class SignupForm(forms.Form):
    username = forms.CharField(max_length=30,
                               min_length=6,
                               label='用户名')
    password = forms.CharField(max_length=30,
                               min_length=6,
                               label='登录密码')
    email = forms.EmailField(label='邮箱地址')

    tos = forms.BooleanField(label='我已阅读并同意服务条款')

    def save(self):
        username, email, password = (self.cleaned_data['username'],
                                     self.cleaned_data['email'],
                                     self.cleaned_data['password'])
        new_user = User.objects.create_user(username, email, password)
        new_user.save()
        return new_user

    def clean_username(self):
        p = re.compile(r'(?!^\d[A-Za-z0-9_-]*$)^[A-Za-z0-9_-]{6,25}$')
        if not p.match(self.cleaned_data['username']):
            raise forms.ValidationError('用户名只允许字母、数字、下划线、横线组成，首位只能为字母，且长度为6-30个字符.')

        if User.objects.filter(username=self.cleaned_data['username']).exists():
            raise forms.ValidationError('这个用户名已经被注册.')

        return self.cleaned_data['username']

    def clean_email(self):
        if User.objects.filter(email__iexact=self.cleaned_data['email']).exists():
            raise forms.ValidationError('这个邮箱已经被注册.')
        return self.cleaned_data['email']


class TopicForm(forms.ModelForm):
    name = forms.CharField(max_length=255, label='主题', required=False, error_messages={ 'max_length': '主题最长为255'})
    captcha = forms.CharField(label='验证码', error_messages={"required": "验证码不能为空"})

    class Meta(object):
        model = Topic
        fields = ('forum', 'topicType', 'name', 'body', 'hiddenreplies', 'ordertype', 'allownoticeauthor', 'usesig', 'reply')

    def __init__(self, *args, **kwargs):
        # Move args to kwargs
        if args:
            kwargs.update(dict(zip(inspect.getargspec(super(TopicForm, self).__init__)[0][1:], args)))
        self.request = kwargs.pop('request', None)
        self.genus_topic = kwargs.pop('topic', None)
        super(TopicForm, self).__init__(**kwargs)

        if self.genus_topic:
            self.fields['forum'].required = False

    def clean(self):
        if self.genus_topic:
            self.cleaned_data['forum'] = self.genus_topic.forum
        return super(TopicForm, self).clean()

    def clean_topicType(self):
        if self.cleaned_data['topicType']:
            if self.cleaned_data['topicType'].forum != self.cleaned_data['forum']:
                raise forms.ValidationError('主题类型错误.')
        return self.cleaned_data['topicType']

    def clean_name(self):
        if self.cleaned_data['name'].strip():
            return self.cleaned_data['name'].strip()
        else:
            if self.genus_topic:
                return "RE:%s"%self.genus_topic.name
            raise forms.ValidationError('主题标题不能为空.')

    def clean_captcha(self):
        ca = Captcha(self.request)
        if not ca.check(self.cleaned_data['captcha']):
            raise forms.ValidationError('验证码错误.')
        return self.cleaned_data['captcha']

    def save(self, commit=True):
        topic = super(TopicForm, self).save(commit=False)
        if self.genus_topic:
            topic.genus_topic = self.genus_topic
        else:
            topic.genus_topic = None
        topic.user = self.request.user
        if commit:
            topic.save()
        return topic

    #     ip = self.request.META.get('REMOTE_ADDR', '')
    #     user = self.request.user
    #     topic = Topic(
    #         forum=self.cleaned_data['forum'],
    #         name=self.cleaned_data['name'],
    #         user=user,
    #         topicType=self.cleaned_data['topicType']
    #     )
    #     topic.save()
    #
    #     return topic

#
# class PostForm(forms.Form):
#     body = forms.CharField(label='内容', error_messages={"required": "内容不能为空"})
#     topic = forms.ModelChoiceField(Topic.objects.all(), label='主题', required=True,
#                                    error_messages={"required": "主题不能为空"})
#     reply = forms.ModelChoiceField(Post.objects.all(), label='主题', required=False)
#     captcha = forms.CharField(label='验证码', error_messages={"required": "验证码不能为空"})
#
#     def clean_reply(self):
#         if self.cleaned_data['reply'].topic != self.cleaned_data['topic']:
#             raise forms.ValidationError('回复的帖子错误.')
#         return self.cleaned_data['reply']
#
#     def clean_captcha(self):
#
#         ca = Captcha(self.request)
#         if not ca.check(self.cleaned_data['captcha']):
#             raise forms.ValidationError('验证码错误.')
#         return self.cleaned_data['captcha']
#
#     def __init__(self, *args, **kwargs):
#         if args:
#             kwargs.update(dict(zip(inspect.getargspec(super(PostForm, self).__init__)[0][1:], args)))
#         self.request = kwargs.pop('request', None)
#         self.topic = kwargs.pop('topic', None)
#         super(PostForm, self).__init__(**kwargs)
#
#     def save(self):
#         ip = self.request.META.get('REMOTE_ADDR', '')
#         user = self.request.user
#         topic = self.cleaned_data['topic']
#         post = Post(topic=topic, user=user,
#                     body=self.cleaned_data['body'],
#                     reply=self.cleaned_data['reply'],
#                     user_ip=ip,
#         )
#         post.save()
#         return post