# -*- coding: utf-8 -*-

import functools
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

# Create your models here.
from django.utils.encoding import python_2_unicode_compatible
from django_extensions.db.fields import CreationDateTimeField
from django.contrib.contenttypes import generic
from bbs import defaults
from bbs.util import get_file_path, summary
from django.db.models.aggregates import Sum


@python_2_unicode_compatible
class Profile(models.Model):
    user = models.OneToOneField(defaults.AUTH_USER_MODEL, verbose_name=_('用户'))

    def __str__(self):
        return self.user.username

    class Meta(object):
        verbose_name = _('用户配置文件')
        verbose_name_plural = _('用户配置文件')


def create_project_topic(sender, instance, created, **kwargs):
    #创建用户配置文件
    if created:
        Profile(user=instance).save()
        permissions = [('change_forum', 'bbs', 'forum'), ('add_forum', 'bbs', 'forum'),
                       ('add_topictype', 'bbs', 'topictype'), ('change_topictype', 'bbs', 'topictype'),
                       ('add_topic', 'bbs', 'topic'), ('change_topic', 'bbs', 'topic'),
                       ('add_topicscore', 'bbs', 'topicscore'),
                       ('add_topiccomment', 'bbs', 'topiccomment'),
                       ('add_report', 'bbs', 'report'),
                       ]

        for p in permissions:
            try:
                permission = Permission.objects.get_by_natural_key(*p)
                instance.user_permissions.add(permission)
            except (Permission.DoesNotExist, ContentType.DoesNotExist):
                pass
        instance.save()

post_save.connect(create_project_topic, sender=defaults.AUTH_USER_MODEL)

@python_2_unicode_compatible
class Category(models.Model):
    name = models.CharField(_('类别名称'), max_length=80)
    sort = models.IntegerField(_('排序'), blank=True, default=0)

    class Meta(object):
        ordering = ['sort']
        verbose_name_plural = _('类别')
        verbose_name = _('类别')

    def __str__(self):
        return self.name

    def forum_count(self):
        return self.forums.all().count()


@python_2_unicode_compatible
class Forum(models.Model):
    category = models.ForeignKey(Category, verbose_name=_('类别'))
    parent = models.ForeignKey('self', related_name='child_forums', verbose_name=_('所属父板块'), blank=True, null=True)
    name = models.CharField(_('板块名称'), max_length=80)
    description = models.TextField(_('描述'), blank=True)
    sort = models.IntegerField(_('排序'), blank=True, default=0)
    moderators = models.ManyToManyField(defaults.AUTH_USER_MODEL, blank=True, null=True, verbose_name=_('版主'))
    updated = models.DateTimeField(_('更新的时间'), blank=True, null=True)
    post_count = models.IntegerField(_('回复数'), blank=True, default=0)
    topic_count = models.IntegerField(_('主题数'), blank=True, default=0)
    icon = models.ImageField('图标', null=True, blank=True,
                             upload_to=functools.partial(get_file_path, to=defaults.PYBB_ATTACHMENT_UPLOAD_TO))

    def get_topics(self):
        """
        :return:该板块下所有的主题
        """
        return self.topic_set.filter(genus_topic__isnull=True)

    @property
    def get_topic_count(self):
        """
        :return: 该板块下的主题数量
        """
        return self.get_topics().count()


    class Meta(object):
        ordering = ['sort']
        verbose_name = _('板块')
        verbose_name_plural = _('板块')

    def __str__(self):
        return self.name




@python_2_unicode_compatible
class TopicType(models.Model):
    name = models.CharField(_('主题类型名称'), max_length=80)
    forum = models.ForeignKey(Forum, verbose_name=_('所属板块'))

    def __str__(self):
        return self.name

    class Meta(object):
        verbose_name = verbose_name_plural = '主题类型'


@python_2_unicode_compatible
class Topic(models.Model):
    forum = models.ForeignKey(Forum, verbose_name=_('板块'))
    name = models.CharField(_('主题标题'), max_length=255)
    body = models.TextField(_('内容'))
    user_ip = models.IPAddressField(_('用户 IP'), blank=True, default='0.0.0.0')

    created = CreationDateTimeField(_("创建的时间"))
    updated = models.DateTimeField(_('更新的时间'), null=True)
    user = models.ForeignKey(defaults.AUTH_USER_MODEL, verbose_name=_('创建的用户'))

    views = models.IntegerField(_('浏览数'), blank=True, default=0)
    topicType = models.ForeignKey(TopicType, null=True, blank=True)

    genus_topic = models.ForeignKey('self', null=True, blank=True, verbose_name="所属主题")

    reply = models.ForeignKey('self', null=True, blank=True, related_name="reply_set", verbose_name="回复的主题")

    #====附加选项
    hiddenreplies = models.BooleanField('回帖仅作者可见', default=False)
    ordertype = models.BooleanField('回帖倒序排列', default=False)
    allownoticeauthor = models.BooleanField('接收回复通知', default=True)
    usesig = models.BooleanField('使用个人签名', default=True)

    def get_reply(self):
        if self.reply:
            if self.reply != self.genus_topic:
                return self.reply

    def get_reply_set(self):
        """
        :return:所有回复
        """
        return self.topic_set.all()

    @property
    def get_reply_count(self):
        """
        取得回复数量
        :return:
        """
        if not getattr(self, '_reply_count', None):
            self._reply_count = self.get_reply_set().count()
        return self._reply_count

    @property
    def last_topic(self):
        """
        :return:最后一次回复
        """
        if not getattr(self, '_last_topic', None):
            try:
                self._last_topic = self.topic_set.order_by('-created', '-id').select_related('user')[0]
            except IndexError:
                self._last_topic = None
        return self._last_topic

    def __str__(self):
        return self.name

    def summary(self):
        return summary(self.body, 30)

    def get_total_score(self):
        return self.topicscore_set.all().aggregate(Sum('money'))['money__sum']

    class Meta(object):
        verbose_name = verbose_name_plural = '主题'


@python_2_unicode_compatible
class TopicScore(models.Model):
    topic = models.ForeignKey(Topic, verbose_name=_('主题'))
    user = models.ForeignKey(defaults.AUTH_USER_MODEL, verbose_name=_('创建的用户'))
    money = models.IntegerField(verbose_name=_('评分的金额'))
    reason = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('评分的理由'))
    created = CreationDateTimeField(_("创建的时间"))

    def __str__(self):
        return self.reason

    class Meta(object):
        verbose_name = verbose_name_plural = '评分'

@python_2_unicode_compatible
class TopicComment(models.Model):
    topic = models.ForeignKey(Topic, verbose_name=_('主题'))
    user = models.ForeignKey(defaults.AUTH_USER_MODEL, verbose_name=_('创建的用户'))
    body = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('点评内容'))
    created = CreationDateTimeField(_("创建的时间"))

    def summary(self):
        return summary(self.body, 30)

    def __str__(self):
        return self.summary()

    class Meta(object):
        verbose_name = verbose_name_plural = '点评'

@python_2_unicode_compatible
class Report(models.Model):
    user = models.ForeignKey(defaults.AUTH_USER_MODEL, verbose_name=_('创建的用户'))
    reason = models.CharField(max_length=255, blank=True, null=True, verbose_name=_('举报的理由'))
    created = CreationDateTimeField(_("创建的时间"))

    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    object_id = models.PositiveIntegerField('object ID', null=True, blank=True)
    content_object = generic.GenericForeignKey()

    def summary(self):
        return summary(self.reason, 30)

    def __str__(self):
        return self.summary()