# -*- coding: utf-8 -*-
from datetime import timedelta

from django import template
from django.template import resolve_variable, VariableDoesNotExist, Context
from django.template.loader import get_template
from django.utils import dateformat
from django.utils.timezone import now as tznow

try:
    import pytils
    pytils_enabled = True
except ImportError:
    pytils_enabled = False

register = template.Library()

@register.tag
def bbs_time(parser, token):
    try:
        tag, context_time = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('bbs_time requires single argument')
    else:
        return BbsTimeNode(context_time)


class BbsTimeNode(template.Node):
    def __init__(self, time):
        self.time = template.Variable(time)

    def render(self, context):
        context_time = self.time.resolve(context)
        if not context_time:
            return ''
        delta = tznow() - context_time
        today = tznow().replace(hour=0, minute=0, second=0)
        yesterday = today - timedelta(days=1)
        tomorrow = today + timedelta(days=1)

        if delta.days == 0:
            if delta.seconds < 60:
                if context['LANGUAGE_CODE'].startswith('ru') and pytils_enabled:
                    msg = '几秒钟前，几秒钟前，秒前'
                    msg = pytils.numeral.choose_plural(delta.seconds, msg)
                else:
                    msg ='秒前'
                return '%d %s' % (delta.seconds, msg)

            elif delta.seconds < 3600:
                minutes = int(delta.seconds / 60)
                if context['LANGUAGE_CODE'].startswith('ru') and pytils_enabled:
                    msg = '分钟前,分钟,分钟前'
                    msg = pytils.numeral.choose_plural(minutes, msg)
                else:
                    msg = '分钟前'
                return '%d %s' % (minutes, msg)
        if today < context_time < tomorrow:
            return '今天, %s' % context_time.strftime('%H:%M')
        elif yesterday < context_time < today:
            return '昨天, %s' % context_time.strftime('%H:%M')
        else:
            return dateformat.format(context_time, 'Y-m-d  H:i')



@register.tag
def bbs_floor(parser, token):
    try:
        tag, index, page, page_size = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('bbs_floor requires single argument')
    else:
        nodelist = parser.parse(('endbbs_floor',))
        parser.delete_first_token()
        return BbsFloorNode(index, page, page_size, nodelist)



class BbsFloorNode(template.Node):
    def __init__(self, index, page, page_size, nodelist):
        self.index = template.Variable(index)
        self.page = template.Variable(page)
        self.page_size = template.Variable(page_size)
        self.nodelist = nodelist

    def render(self, context):
        index = int(self.index.resolve(context))
        try:
            page = int(self.page.resolve(context))-1
        except VariableDoesNotExist:
            page = 0
        page_size = int(self.page_size.resolve(context))
        value = {'floor': index+(page*page_size)}
        context.update(value)
        output = self.nodelist.render(context)
        context.pop()
        return output


@register.tag
def bbs_user(parser, token):
    try:
        tag, user, m_user = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError('bbs_user requires single argument')
    else:
        return BbsUserNode(user, m_user)


class BbsUserNode(template.Node):
    def __init__(self, user, m_user):
        self.user = user
        self.m_user = m_user

    def render(self, context):
        user = resolve_variable(self.user, context)
        m_user = resolve_variable(self.m_user, context)
        _template = get_template("bbs/include/user_info_pop.html")
        user_action = False
        if m_user.is_authenticated() and m_user.id != user.id:
            user_action = True
        output = _template.render(Context({"user": user, 'user_action':user_action}))
        return output